#! /usr/bin/python3
## This script runs on a Rapberry pi 3+, and activates the "holidays mode" of an ELCO heat pump (PAC in french)
## Goal is to avoid having the heat pump beeing active during holidays, while keeping the anti-frost mode active
## This is done by suitably conencting a electromechanical relay between H1 and M contacts in the pump regulation
## This relay as well as LED 7 LCD are driven by this simple software
##
## Written by Yves OESCH, August 2016
##
## History:
## PAC_01.py: First working prototype. Bug: after 2d, first press ==> 2 days increase. Not cofirmed
## PAC_02.py: PAC activation only after entering slow GUI mode. Cleanup, Comments
## PAC_03.py: Added soft OFF button

import RPi.GPIO as GPIO
import time
import Adafruit_CharLCD as LCD
from queue import Queue
import datetime
from subprocess import call
import os
print('Imports OK')

fast_GUI_timeout=5	#Timeout between fast and slow GUI. PAC relay/LED update, backlight [s]
debounce_time=200	#Keyboard debouncing time [ms]
lcd_refresh_delay_fast=0.1  #refresh delay, fast GUI [s] (user behind GUI, backlight ON)
lcd_refresh_delay_slow=2    #refresh delay, slow GUI [s] (User away, backlight OFF)


#Pinout (BCM mode)
button_clear  = 5
button_plus   = 6
button_shutdn = 13
led_pin	      = 17	#Display PAC status (HIGH=PAC OFF/RED LED/Relay active; LOW: PAC ON,Green LED, Relay inactive
lcd_rs        = 27      # Note this might need to be changed to 21 for older revision Pi's.
lcd_en        = 22
lcd_d4        = 25
lcd_d5        = 24
lcd_d6        = 23
lcd_d7        = 18
lcd_backlight = 12

# Define LCD column and row size for 16x2 LCD.
lcd_columns   = 16
lcd_rows      = 2



GPIO.setmode(GPIO.BCM)                                      #Broadcom numbering scheme for GPIO (not header)
GPIO.setup(button_clear, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(button_plus, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(button_shutdn, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(led_pin, GPIO.OUT)
GPIO.output(led_pin,GPIO.LOW)                               #Set PAC_ON (normal mode) at startup



# Initialize the LCD using the pins above.
lcd = LCD.Adafruit_CharLCD(lcd_rs, lcd_en, lcd_d4, lcd_d5, lcd_d6, lcd_d7,
                           lcd_columns, lcd_rows, lcd_backlight,False)
lcd.clear()
lcd.show_cursor(False)
lcd.message('Boot')
time.sleep(1)


FAST_GUI=True	                        #Describe the reactivity of the GUI. True when recent user input
PAC_ON=True	                        #Status of the PAC
LastButtonPress=datetime.datetime.now() #Stores last button press time to switch GUI speed
q=Queue()                               #Stores key presses, filled in another thread
PacRestartDay=datetime.date.today()     #Stores the PAC restart day (assumed restart time is 00:00:00)

def ButtonClearPressed(channel):
    #print('Button ''CLEAR'' pressed')
    #print(channel)
    q.put('CLEAR')
    

def ButtonPlusPressed(channel):
    #print('Button ''PLUS'' pressed')
    #print(channel)
    q.put('PLUS')

def ButtonShutdnPressed(channel):
    #os.system('sudo halt')#,shell=False)
    q.put('SHUTDOWN')

GPIO.add_event_detect(button_clear,GPIO.FALLING,callback=ButtonClearPressed,bouncetime=debounce_time)
GPIO.add_event_detect(button_plus,GPIO.FALLING,callback=ButtonPlusPressed,bouncetime=debounce_time)
GPIO.add_event_detect(button_shutdn,GPIO.FALLING,callback=ButtonShutdnPressed,bouncetime=debounce_time)
try:
    while True:
            #print(q.qsize())                                #Read button queue and update PAC start time
            if q.qsize()>0:
                item=q.get()
                if item is None:
                    break
                print(item)
                if item=='CLEAR':                           #reset the PAC restard date to today
                    PacRestartDay=datetime.date.today()
                if item=='PLUS':                            # increment the PAC restart date by one full day
                    PacRestartDay=PacRestartDay+datetime.timedelta(1)
                if item=='SHUTDOWN':                        # rather brutal shutdown on button press
                    GPIO.output(led_pin,GPIO.LOW)           # Switch PAC in normal mode again
                    lcd.clear()
                    message='Wait 30 seconds\nand remove power'
                    print(message)
                    lcd.message(message)
                    lcd.set_backlight(0)
                    #time.sleep(5)
                    os.system('sudo halt')

                LastButtonPress=datetime.datetime.now()     #To evaluate if switching to slow GUI is needed (timeout)
                #print(PacRestartDay)
                q.task_done()
            
            now=datetime.datetime.now()
            print('now: ',now)
            if LastButtonPress+datetime.timedelta(0,fast_GUI_timeout)<now: #Last button press is older than (days,seconds)
                FAST_GUI=False
                lcd_refresh_delay=lcd_refresh_delay_slow
                lcd.set_backlight(0)
            else:                                           #User still "behind" user interface
                FAST_GUI=True
                lcd_refresh_delay=lcd_refresh_delay_fast
                lcd.set_backlight(1)
            print('Fast GUI:',FAST_GUI)
            print('PAC_ON: ',PAC_ON)
            
            today=datetime.date.today()
            print('today: ',today)
            print('PAC restart day: ',PacRestartDay)
            #print('strftime: ', today.strftime('%d.%m.%Y'))
            if PacRestartDay>today:                         #PAC restart in the future
                    print('PacRestartDay>today')
                    PAC_ON=False
            else:
                    print('PacRestartDay<=today')           # PAC restart present or past
                    PAC_ON=True
                    PacRestartDay=today

            if PAC_ON:                                      #Normal mode
                    print('update lcd PAC_ON')
                    lcd.clear()
                    message='Date actuelle:\n'+today.strftime('%d.%m.%Y')
                    print(message)
                    lcd.message(message)
                    if not(FAST_GUI):                       #avoid sending contradictory message to PAC
                        GPIO.output(led_pin,GPIO.LOW)       #if user toggles back and forth ON/OFF
                        print('LED low')

            else: #if not(PAC_ON): #instead of PAC_ON==False:   #Hollidays mode
                   print('update lcd PAC_OFF')
                   lcd.clear()
                   message='Demarrage PAC:\n'+PacRestartDay.strftime('%d.%m.%Y')
                   print(message)
                   lcd.message(message)
                   if not(FAST_GUI):
                        GPIO.output(led_pin,GPIO.HIGH)
                        print('LED high')
            time.sleep(lcd_refresh_delay)
            print('\n')
                
             
            
except:                                                     #Ctrl-C in terminal window
    lcd.clear()
    lcd.message('THE END')
    GPIO.cleanup()
    print('******************* THE END ***************************')
            
    
